Texture1D<float4> IniParams : register(t120);

struct vs2ps {
	float4 pos : SV_Position0;
	float2 uv : TEXCOORD1;
};

#ifdef VERTEX_SHADER

#define wh IniParams[2].xy

void main(
		out vs2ps output,
		uint vertex : SV_VertexID)
{
	switch(vertex) {
		case 0:
			output.pos.xy = float2(2.0 / (wh.x / wh.y) - 1.0, 1);
			output.uv = float2(0,1);
			break;
		case 1:
			output.pos.xy = float2(2.0 / (wh.x / wh.y) - 1.0, -1);
			output.uv = float2(0,0);
			break;
		case 2:
			output.pos.xy = float2(-1, 1);
			output.uv = float2(1,1);
			break;
		case 3:
			output.pos.xy = float2(-1, -1);
			output.uv = float2(1,0);
			break;
		default:
			output.pos.xy = 0;
			output.uv = float2(0,0);
			break;
	};
	output.pos.zw = float2(0, 1);
}
#endif

#ifdef PIXEL_SHADER

Texture2D<float4> MenuCell : register(t0);

Texture2DArray<float4> ArrayPoints : register(t1);

Texture2D<float4> KCOne : register(t2);

Texture2D<float4> InfoImg : register(t4);

Texture2D<float4> ButtonWB : register(t5);

Texture2DArray<float4> Buttons : register(t6);

Buffer<float4> Point : register(t3);

#define kControl IniParams[0].xy
#define Info IniParams[1].x
#define Size IniParams[1].y
#define Slot IniParams[2].z

SamplerState s0_s : register(s0);

void main(vs2ps input, out float4 result : SV_Target0)
{	
	float3 colors[8] = {
		float3(1.75, 0.34, 0.34),     
		float3(0.51, 1.35, 0.52),     
		float3(0.33, 1.01, 1.33),
		float3(1.75, 1.75, 0.55),
		float3(1.75, 1, 0.55),
		float3(0.82, 0.54, 1.75),
		float3(0, 1, 1),    
		float3(1, 0.5, 0.8)
	};
	
	float3 blend;
	
	input.uv.y = 1 - input.uv.y;
	input.uv.x = 1 - input.uv.x;
		
	int col = input.uv.x / 0.03125;
	int row = input.uv.y / 0.03125;
	int index = (row * 32) + col;
	
	result = float4(0,0,0,0);
	
	uint PointState;
	uint PointXYZW = index / 4.0;
	switch(index % 4.0)
	{
	case 0: 
		PointState = Point[PointXYZW].x;
		break;
	case 1: 
		PointState = Point[PointXYZW].y;
		break;
	case 2: 
		PointState = Point[PointXYZW].z;
		break;
	case 3: 
		PointState = Point[PointXYZW].w;
		break;
	default:
		PointState = 0;
		break;
	}
	
	
	if (index < 494){
		result = MenuCell.Sample(s0_s, input.uv.xy * 32.0);
		
		if (PointState == 1) result.xyz = result.xyz * colors[Slot];
				
		float4 Layer1 = ArrayPoints.Sample(s0_s, float3(input.uv.xy * 32.0,index));
		blend = smoothstep(0.0, 1.0, Layer1.w);
		result.xyz = lerp(result.xyz, Layer1.xyz, blend);
		
		if (PointState == 1){
			float4 CellFrame = KCOne.Sample(s0_s, input.uv.xy * 32.0);
			blend = smoothstep(0.0, 1.0, CellFrame.w);
			result.xyz = lerp(result.xyz, CellFrame.xyz * colors[Slot], blend);
		}
		
		if (kControl.x == 1 && kControl.y == index) {
			result.xyz = float3(1,1,1) - result.xyz;
			float4 Select = KCOne.Sample(s0_s, input.uv.xy * 32.0);
			blend = smoothstep(0.0, 1.0, Select.w);
			result.xyz = lerp(result.xyz, Select.xyz * colors[Slot], blend);
			if (result.w != 1) result.w = result.w + Select.w;
			if (result.w > 1) result.w = 1;
		}
	}
	
	result.xyz = exp2(log2(result.xyz) * 2.2 / 1.0);
	
	if (row == 29) {
		switch(col)
		{
			case 0:
				switch(Size)
				{
					case 1.0:
						result = Buttons.Sample(s0_s, float3(input.uv.xy * 32.0,16));
						break;
					case 2.0:
						result = Buttons.Sample(s0_s, float3(input.uv.xy * 32.0,17));
						break;
					case 4.0:
						result = Buttons.Sample(s0_s, float3(input.uv.xy * 32.0,18));
						break;
					case 8.0:
						result = Buttons.Sample(s0_s, float3(input.uv.xy * 32.0,19));
						break;
					default: 
						break;
				}
				break;
			case 1:
				if (Size == 1.0) result = Buttons.Sample(s0_s, float3(input.uv.xy * 32.0,5));
				else result = Buttons.Sample(s0_s, float3(input.uv.xy * 32.0,4));
				break;
			case 2: 
				if (Size == 2.0) result = Buttons.Sample(s0_s, float3(input.uv.xy * 32.0,7));
				else result = Buttons.Sample(s0_s, float3(input.uv.xy * 32.0,6));
				break;
			case 3: 
				if (Size == 4.0) result = Buttons.Sample(s0_s, float3(input.uv.xy * 32.0,9));
				else result = Buttons.Sample(s0_s, float3(input.uv.xy * 32.0,8));
				break;
			case 4: 
				if (Size == 8.0) result = Buttons.Sample(s0_s, float3(input.uv.xy * 32.0,11));
				else result = Buttons.Sample(s0_s, float3(input.uv.xy * 32.0,10));
				break;
			case 30: 
				result = Buttons.Sample(s0_s, float3(input.uv.xy * 32.0,0));
				break;
			case 31: 
				result = Buttons.Sample(s0_s, float3(input.uv.xy * 32.0,2));
				break;
			default: 
				break;
		}
	}
	
	if (row > 29) {
		int colorB = col/4.0;
		if (colorB == Slot) result = ButtonWB.Sample(s0_s, input.uv.xy * 8);
		else result = ButtonWB.Sample(s0_s, input.uv.xy * 8 + float2(0,0.5));
		result.xyz = result.xyz * colors[colorB];
	}
	
	result.w = result.w + 0.5;
	
	if (Info == 1) {
		float4 InfoLayer = InfoImg.Sample(s0_s, input.uv.xy);
		if (InfoLayer.w != 0) result.w = 1;
		blend = smoothstep(0.0, 1.0, InfoLayer.w);
		result.xyz = lerp(result.xyz, InfoLayer.xyz, blend);
	}
	
}
#endif
